import es
import popuplib
import gamethread

import threading
import os
import urllib2
from configobj import ConfigObj


#plugin information
info = es.AddonInfo() 
info.name     = 'Addon Utils' 
info.version  = '0.5.0' 
info.url      = 'http://addons.eventscripts.com/addons/view/addon_utils' 
info.basename = 'addon_utils' 
info.author   = 'Errant'

'''
Addon Utils is a companion plugin for es_install and a general plugin information viewer

== Features ==
* Anyone can view a list (aand details) of the addons loaded on the server
* Admins can unload loaded addons with just a few button presses
* Detailed information about every addon
* Addon info is kept up to date using Wget - no laggy threading!

== Coming Soon ==
* es_install support (auto install updates!)
* Remote notifications (recieve important news straight into game)
* Optional support for Auth service
* In game admin menus
* Detailed addons installtion logs
* Support for UNLOADED addons (load or update them)
* Common addons list to install from
* "Clever" version # parsing - better than ever at guessing which plugin versins are which
* Even more detailed addon info
* Multi language support
 
== Changelog ==
 ** V0.5 **
  - Prerelease / Tech Preview build
  - Support for python addons
  - Supports basic addon info and unloading
  - Updates addon versions on map start
  - No admin menu
  
Notes for me:
 - Move to classes for the addons (easier!)
 - Figure out a better update method
'''

versions_path = es.getAddonPath(info.basename) + "/versions"
log_path = es.getAddonPath(info.basename) + "/logs"

admins = ["STEAM_0:1:8604050"]
# sort out wget
conwget="wget"
if os.name == "nt":
    conwget = es.getAddonPath('addon_utils') + "/wget/wget.exe"
    

def load():
    
    if not os.path.exists(versions_path):
        os.mkdir(versions_path)
    if not os.path.exists(log_path):
        os.mkdir(log_path)
    # touch all the files too
    if not os.path.exists(log_path+"/wget_log.txt"):
        open(log_path+"/wget_log.txt","w").close()
    if not os.path.exists("%s/%s.txt" % (log_path,str(info.version))):
        open("%s/%s.txt" % (log_path,str(info.version)),"w").close()
    # 5 seconds delay then run a quick update of the stats
    # delayed so we know everythign is loaded :D
    gamethread.delayed(5,_update_remote_versions,())
    es.regsaycmd("!addons","addon_utils/addons_cmd")
    
def unload():
    es.unregsaycmd("!addons")
    
def es_map_start(ev):
    _update_remote_versions()
    
def addons_cmd():
    send_main_menu(es.getcmduserid())
    
def _get_local_plugins():
    '''
    All the local py plugins and their versions
    '''
    ret = {}
    for x in es.addons.AddonList:
        if x.__dict__.has_key("info"):
            xname = x.__name__[x.__name__.rfind(".")+1:]  # nicked from es.py :P
            info = x.__dict__["info"]
            ret[xname]= {"version":info.version,"importname":x.__name__,"info":info}
    return ret

def _update_remote_versions(type="threaded"):
    '''
    
    '''
    addons=_get_local_plugins()
    for x in addons:
        # sort out a basename to use
        basename = str(x)
        if addons[x]["info"].has_key("basename"):
            basename = addons[x]["info"].basename
        # set up some vars for the wget command
        wget_log =  "%s/wget_log.txt" % log_path
        spath = "%s/%s.txt" % (versions_path,basename)
        rpath = "http://addons.eventscripts.com/addons/chklatestver/%s" % basename
        # wget the data
        os.system(conwget + " -b -t 5 -a %s -O %s %s" % (wget_log,spath,rpath))

def send_main_menu(userid):
    mMenu = popuplib.create("au_main_menu")
    mMenu.menuselect = _menu_handler
    mMenu.addline(" ++ Addon Utils [V%s] ++ " % info.version)
    mMenu.addline("  By Errant")
    mMenu.addline(" ")
    mMenu.addline("->1. List Addons")
    mMenu.addline("->2. About Addon Utils")
    mMenu.addline(" ")
    if es.getplayersteamid(userid) in admins:
        mMenu.addline("->3. Settings (admin only)")
        mMenu.addline("->4. Add Admin (admin only)")
    mMenu.addline(" ")
    mMenu.addline("->0. Exit")
    mMenu.send(userid)
    
    
    
def send_version_menu(userid):
    '''
    * Builds a menu of all your PYTHON plugins and checks their version
    '''
    
    # make the menu!
    vMenu = popuplib.easymenu("plugin_versions_menu",None,_menu_handler)
    vMenu.settitle("Select an Addon for Options")
    addons = _get_local_plugins()
    # go through ach addon and add them to the list
    for x in addons:
        # get the info
        basename, thisver, cver, outofdate = parse_plugin(addons[x],str(x))
        # build the menu option
        vMenu.addoption(x,"%s%s (%s)" % (outofdate,basename,thisver))
    vMenu.send(userid)
    
def parse_plugin(addon,aname):
    outofdate=""
    thisver = str(addon["info"].version)
    # sort out a basename to use
    basename = aname
    if addon["info"].has_key("basename"):
        basename = addon["info"].basename
    vpath = "%s/%s.txt" % (versions_path,basename)
    if os.path.exists(vpath):
        cver = open(vpath,"r").read()
        if not cver:
            cver = "Unkown"
    else:
        cver - "Unknown"
    cver = str(cver)
    if cver != "Unknown":
        # time to compare the versions!
        # for now lets simply do != 
        if cver != thisver:
            # not the same
            outofdate="[OLD] "
    return basename, thisver, cver, outofdate
    
def _menu_handler(userid, choice, popupid):
    if popupid == "au_main_menu":
        if choice == 1:
            send_version_menu(userid)
        if choice == 2:
            # send them about plugin popup
            abtMenu = popuplib.create("about")
            abtMenu.addline(" ++ Addon Utils ++ ")
            abtMenu.addline(" ")
            abtMenu.addline("Author: Errant")
            abtMenu.addline("Version: %s" % info.version)
            abtMenu.addline("Addon Utils is a small plugin to")
            abtMenu.addline("help you manage ES server addons.")
            abtMenu.addline(" ")
            abtMenu.addline("It lets anyone view which addons")
            abtMenu.addline("are installed, and lets admins update,")
            abtMenu.addline("unload and otherwise manage them.")
            abtMenu.addline(" ")
            abtMenu.addline("->0. Close ")
            abtMenu.send(userid)
        if choice == 3:
            if es.getplayersteamid(userid) in admins:
                # send them the admin menu
                es.tell(userid, "#multi", "#green[Addon Utils]#lightgreen Sorry no admin menu yet. V1.0+ will include these options")
            else:
                # oopsie
                pass
        if choice == 4:
            if es.getplayersteamid(userid) in admins:
                # send them the admin menu
                es.tell(userid, "#multi", "#green[Addon Utils]#lightgreen Sorry no admin menu yet. V1.0+ will include these options")
            else:
                # oopsie
                pass
    if popupid == "plugin_versions_menu":
        addons = _get_local_plugins()
        if choice in addons:
            # grab the data
            basename, thisver, cver, outofdate = parse_plugin(addons[choice],choice)
            # send the further info menu
            aMenu = popuplib.create("plugin_info_menu-%s" % choice)
            aMenu.menuselect = _menu_handler
            aMenu.addline(" == Info for %s ==" % basename)
            if outofdate == "":
                # its up to date
                aMenu.addline(" ** %s is up to date ** " % basename)
                aMenu.addline(" ")
                aMenu.addline("Current Version: %s" % thisver)
            elif cver == "Unknown":
                # probably not on ESAM
                aMenu.addline(" ** Unable to find %s on ESAM ** " % basename)
                aMenu.addline(" ")
                aMenu.addline("Current Version: %s" % thisver)
            else:
                aMenu.addline(" ** %s is out of date ** " % basename)
                aMenu.addline("->1. Install the latest version")
                aMenu.addline(" ")
                aMenu.addline("Current Version: %s" % thisver)
                aMenu.addline("ESAM Version: %s" % cver)
                aMenu.addline(" ")
            pinfo = addons[choice]["info"]
            if pinfo.has_key("author"):
                aMenu.addline("Author: %s" % pinfo.author)
            if pinfo.has_key("name"):
                # name may contain some more info
                if pinfo.name != basename:
                    aMenu.addline("Desc: %s " % pinfo.name)
            aMenu.addline(" ")
            if es.getplayersteamid(userid) in admins:
                aMenu.addline("->2. Unload %s" % basename)
                aMenu.addline(" ")
            aMenu.addline("->0. Exit")
            aMenu.send(userid)
        else:
            # oops its gone!
            pass
    if str(popupid).startswith("plugin_info_menu"): 
        if es.getplayersteamid(userid) in admins:
            addon = popupid.split("-")[1]
            addons = _get_local_plugins()
            if addon in addons:
                # ok go!
                if choice == 1:
                    # es_install
                    basename, thisver, cver, outofdate = parse_plugin(addons[addon],addon)
                    iMenu = popuplib.create("install_addon")
                    iMenu.addline(" ++ Installing %s ++" % basename)
                    iMenu.addline(" ")
                    iMenu.addline("Sorry auto-installation is disabled")
                    iMenu.addline("at this time. Once es_install is")
                    iMenu.addline("released this option will become")
                    iMenu.addline("available.")
                    iMenu.addline(" ")
                    iMenu.addline("In the mean time you can download")
                    iMenu.addline("this addon manually from ESAM.")
                    iMenu.addline(" ")
                    iMenu.addline("->0. Close ")
                    iMenu.send(userid)
                if choice == 2:
                    uMenu = popuplib.create("unload_%s" % addon)
                    uMenu.addline("Are you sure you want to unload %s" % addon)
                    uMenu.addline("->1. Yes")
                    uMenu.addline("->2. No")
                    uMenu.menuselect=_menu_handler
                    uMenu.send(userid)
                    
        else:
            # you have to be admin!
            pass
    if popupid.startswith("unload_") and es.getplayersteamid(userid) in admins:
        addon = popupid.split("_")[1]
        es.unload(addon)
        es.tell(userid, "#multi", "#green[Addon Utils]#lightgreen %s was unloaded" % addon)